@@ -418,7 +418,7 @@ bool savemeals(FILE * handle,uint32_t starttime,uint32_t endtime) {
 #endif
 
 #ifdef JUGGLUCO_APP
-extern bool libreviewexport(int handle,uint32_t starttime,uint32_t endtime)  ;
+extern bool libreviewexport(int handle,uint32_t starttime,uint32_t endtime,const bool calibrate)  ;
 bool  exportdata(uint32_t starttimein, uint32_t duration,int intype,int fd,float days) {
 	uint32_t endtime=std::min(starttimein+duration,(uint32_t)time(nullptr));
 	uint32_t starttime=endtime-days*24*60*60;
@@ -432,7 +432,7 @@ bool  exportdata(uint32_t starttimein, uint32_t duration,int intype,int fd,float
 #ifdef USE_MEAL
 		case 4: return allsavemeals(fd,starttime,endtime);
 #endif
-		case 5: return libreviewexport(fd,starttime,endtime);
+		case 5: return libreviewexport(fd,starttime,endtime,calibrated);
 		};
 	return false;
 	}
